@@ -4,7 +4,7 @@
  * Copyright (C) 2003 Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,7 +12,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
@@ -147,7 +147,7 @@ fold_lines (char *buf)
 					p = g_utf8_next_char (next2);
 				}
 				else {
-					str = g_string_append (str, CRLF);
+					g_string_append (str, CRLF);
 					p = g_utf8_next_char (next);
 				}
 			}
@@ -155,12 +155,12 @@ fold_lines (char *buf)
 				p = g_utf8_next_char (next);
 			}
 			else {
-				str = g_string_append (str, CRLF);
+				g_string_append (str, CRLF);
 				p = g_utf8_next_char (p);
 			}
 		}
 		else {
-			str = g_string_append_unichar (str, g_utf8_get_char (p));
+			g_string_append_unichar (str, g_utf8_get_char (p));
 			p = g_utf8_next_char (p);
 		}
 	}
@@ -235,19 +235,23 @@ read_attribute_value (EVCardAttribute *attr, char **p, gboolean quoted_printable
 				 * 2 kinds of line folding
 				 */
 			}
-			else if (isalnum(a) && isalnum (b)) {
-				char c;
+			else if (isxdigit(a) && isxdigit (b)) {
+				gunichar c;
 
 				a = tolower (a);
 				b = tolower (b);
 
 				c = (((a>='a'?a-'a'+10:a-'0')&0x0f) << 4)
 					| ((b>='a'?b-'a'+10:b-'0')&0x0f);
-
-				str = g_string_append_c (str, c);
+				
+				g_string_append_unichar (str, c);
 			}
-			/* silently consume malformed input, and
-			   continue parsing */
+			else 
+				{
+					g_string_append_c (str, a);
+					g_string_append_c (str, b);
+				}
+			
 			lp++;
 		}
 		else if (*lp == '\\') {
@@ -255,19 +259,19 @@ read_attribute_value (EVCardAttribute *attr, char **p, gboolean quoted_printable
 			   the characters */
 			lp = g_utf8_next_char(lp);
 			if (*lp == '\0') {
-				str = g_string_append_c (str, '\\');
+				g_string_append_c (str, '\\');
 				break;
 			}
 			switch (*lp) {
-			case 'n': str = g_string_append_c (str, '\n'); break;
-			case 'r': str = g_string_append_c (str, '\r'); break;
-			case ';': str = g_string_append_c (str, ';'); break;
-			case ',': str = g_string_append_c (str, ','); break;
-			case '\\': str = g_string_append_c (str, '\\'); break;
+			case 'n': g_string_append_c (str, '\n'); break;
+			case 'r': g_string_append_c (str, '\r'); break;
+			case ';': g_string_append_c (str, ';'); break;
+			case ',': g_string_append_c (str, ','); break;
+			case '\\': g_string_append_c (str, '\\'); break;
 			default:
 				g_warning ("invalid escape, passing it through");
-				str = g_string_append_c (str, '\\');
-				str = g_string_append_unichar (str, g_utf8_get_char(lp));
+				g_string_append_c (str, '\\');
+				g_string_append_unichar (str, g_utf8_get_char(lp));
 				break;
 			}
 			lp = g_utf8_next_char(lp);
@@ -279,7 +283,7 @@ read_attribute_value (EVCardAttribute *attr, char **p, gboolean quoted_printable
 			lp = g_utf8_next_char(lp);
 		}
 		else {
-			str = g_string_append_unichar (str, g_utf8_get_char (lp));
+			g_string_append_unichar (str, g_utf8_get_char (lp));
 			lp = g_utf8_next_char(lp);
 		}
 	}
@@ -310,7 +314,7 @@ read_attribute_params (EVCardAttribute *attr, char **p, gboolean *quoted_printab
 			lp = g_utf8_next_char (lp);
 		}
 		else if (in_quote || g_unichar_isalnum (g_utf8_get_char (lp)) || *lp == '-' || *lp == '_') {
-			str = g_string_append_unichar (str, g_utf8_get_char (lp));
+			g_string_append_unichar (str, g_utf8_get_char (lp));
 			lp = g_utf8_next_char (lp);
 		}
 		/* accumulate until we hit the '=' or ';'.  If we hit
@@ -490,7 +494,7 @@ read_attribute (char **p)
 			}
 		}
 		else if (g_unichar_isalnum (g_utf8_get_char (lp)) || *lp == '-' || *lp == '_') {
-			str = g_string_append_unichar (str, g_utf8_get_char (lp));
+			g_string_append_unichar (str, g_utf8_get_char (lp));
 		}
 		else {
 			g_warning ("invalid character found in attribute group/name");
@@ -516,6 +520,8 @@ read_attribute (char **p)
 		/* skip past the ';' */
 		lp = g_utf8_next_char(lp);
 		read_attribute_params (attr, &lp, &is_qp);
+		if (is_qp)
+			attr->encoding = EVC_ENCODING_RAW;
 	}
 	if (*lp == ':') {
 		/* skip past the ':' */
@@ -591,6 +597,14 @@ parse (EVCard *evc, const char *str)
 	g_free (buf);
 }
 
+/**
+ * e_vcard_escape_string:
+ * @s: the string to escape
+ *
+ * Escapes a string according to RFC2426, section 5.
+ *
+ * Return value: A newly allocated, escaped string.
+ **/
 char*
 e_vcard_escape_string (const char *s)
 {
@@ -603,24 +617,24 @@ e_vcard_escape_string (const char *s)
 	for (p = s; p && *p; p++) {
 		switch (*p) {
 		case '\n':
-			str = g_string_append (str, "\\n");
+			g_string_append (str, "\\n");
 			break;
 		case '\r':
 			if (*(p+1) == '\n')
 				p++;
-			str = g_string_append (str, "\\n");
+			g_string_append (str, "\\n");
 			break;
 		case ';':
-			str = g_string_append (str, "\\;");
+			g_string_append (str, "\\;");
 			break;
 		case ',':
-			str = g_string_append (str, "\\,");
+			g_string_append (str, "\\,");
 			break;
 		case '\\':
-			str = g_string_append (str, "\\\\");
+			g_string_append (str, "\\\\");
 			break;
 		default:
-			str = g_string_append_c (str, *p);
+			g_string_append_c (str, *p);
 			break;
 		}
 	}
@@ -628,6 +642,14 @@ e_vcard_escape_string (const char *s)
 	return g_string_free (str, FALSE);
 }
 
+/**
+ * e_vcard_unescape_string:
+ * @s: the string to unescape
+ *
+ * Unescapes a string according to RFC2426, section 5.
+ *
+ * Return value: A newly allocated, unescaped string.
+ **/
 char*
 e_vcard_unescape_string (const char *s)
 {
@@ -643,19 +665,19 @@ e_vcard_unescape_string (const char *s)
 		if (*p == '\\') {
 			p++;
 			if (*p == '\0') {
-				str = g_string_append_c (str, '\\');
+				g_string_append_c (str, '\\');
 				break;
 			}
 			switch (*p) {
-			case 'n':  str = g_string_append_c (str, '\n'); break;
-			case 'r':  str = g_string_append_c (str, '\r'); break;
-			case ';':  str = g_string_append_c (str, ';'); break;
-			case ',':  str = g_string_append_c (str, ','); break;
-			case '\\': str = g_string_append_c (str, '\\'); break;
+			case 'n':  g_string_append_c (str, '\n'); break;
+			case 'r':  g_string_append_c (str, '\r'); break;
+			case ';':  g_string_append_c (str, ';'); break;
+			case ',':  g_string_append_c (str, ','); break;
+			case '\\': g_string_append_c (str, '\\'); break;
 			default:
 				g_warning ("invalid escape, passing it through");
-				str = g_string_append_c (str, '\\');
-				str = g_string_append_unichar (str, g_utf8_get_char(p));
+				g_string_append_c (str, '\\');
+				g_string_append_unichar (str, g_utf8_get_char(p));
 				break;
 			}
 		}
@@ -674,12 +696,28 @@ e_vcard_construct (EVCard *evc, const char *str)
 		parse (evc, str);
 }
 
+/**
+ * e_vcard_new:
+ * 
+ * Creates a new, blank #EVCard.
+ *
+ * Return value: A new, blank #EVCard.
+ **/
 EVCard *
 e_vcard_new ()
 {
 	return e_vcard_new_from_string ("");
 }
 
+/**
+ * e_vcard_new_from_string:
+ * @str: a string representation of the vcard to create
+ *
+ * Creates a new #EVCard from the passed-in string
+ * representation.
+ *
+ * Return value: A new #EVCard.
+ **/
 EVCard *
 e_vcard_new_from_string (const char *str)
 {
@@ -709,12 +747,12 @@ e_vcard_to_string_vcard_30 (EVCard *evc)
 
 	GString *str = g_string_new ("");
 
-	str = g_string_append (str, "BEGIN:VCARD" CRLF);
+	g_string_append (str, "BEGIN:VCARD" CRLF);
 
 	/* we hardcode the version (since we're outputting to a
 	   specific version) and ignore any version attributes the
 	   vcard might contain */
-	str = g_string_append (str, "VERSION:3.0" CRLF);
+	g_string_append (str, "VERSION:3.0" CRLF);
 
 	for (l = evc->priv->attributes; l; l = l->next) {
 		GList *p;
@@ -733,10 +771,10 @@ e_vcard_to_string_vcard_30 (EVCard *evc)
 		 */
 
 		if (attr->group) {
-			attr_str = g_string_append (attr_str, attr->group);
-			attr_str = g_string_append_c (attr_str, '.');
+			g_string_append (attr_str, attr->group);
+			g_string_append_c (attr_str, '.');
 		}
-		attr_str = g_string_append (attr_str, attr->name);
+		g_string_append (attr_str, attr->name);
 
 		/* handle the parameters */
 		for (p = attr->params; p; p = p->next) {
@@ -744,10 +782,10 @@ e_vcard_to_string_vcard_30 (EVCard *evc)
 			/* 5.8.2:
 			 * param        = param-name "=" param-value *("," param-value)
 			 */
-			attr_str = g_string_append_c (attr_str, ';');
-			attr_str = g_string_append (attr_str, param->name);
+			g_string_append_c (attr_str, ';');
+			g_string_append (attr_str, param->name);
 			if (param->values) {
-				attr_str = g_string_append_c (attr_str, '=');
+				g_string_append_c (attr_str, '=');
 				for (v = param->values; v; v = v->next) {
 					char *value = v->data;
 					char *p = value;
@@ -760,17 +798,17 @@ e_vcard_to_string_vcard_30 (EVCard *evc)
 						p = g_utf8_next_char (p);
 					}
 					if (quotes)
-						attr_str = g_string_append_c (attr_str, '"');
-					attr_str = g_string_append (attr_str, value);
+						g_string_append_c (attr_str, '"');
+					g_string_append (attr_str, value);
 					if (quotes)
-						attr_str = g_string_append_c (attr_str, '"');
+						g_string_append_c (attr_str, '"');
 					if (v->next)
-						attr_str = g_string_append_c (attr_str, ',');
+						g_string_append_c (attr_str, ',');
 				}
 			}
 		}
 
-		attr_str = g_string_append_c (attr_str, ':');
+		g_string_append_c (attr_str, ':');
 
 		for (v = attr->values; v; v = v->next) {
 			char *value = v->data;
@@ -778,15 +816,15 @@ e_vcard_to_string_vcard_30 (EVCard *evc)
 
 			escaped_value = e_vcard_escape_string (value);
 
-			attr_str = g_string_append (attr_str, escaped_value);
+			g_string_append (attr_str, escaped_value);
 			if (v->next) {
 				/* XXX toshok - i hate you, rfc 2426.
 				   why doesn't CATEGORIES use a ; like
 				   a normal list attribute? */
 				if (!strcmp (attr->name, "CATEGORIES"))
-					attr_str = g_string_append_c (attr_str, ',');
+					g_string_append_c (attr_str, ',');
 				else
-					attr_str = g_string_append_c (attr_str, ';');
+					g_string_append_c (attr_str, ';');
 			}
 
 			g_free (escaped_value);
@@ -798,25 +836,40 @@ e_vcard_to_string_vcard_30 (EVCard *evc)
 		 */
 		l = 0;
 		do {
-			if (attr_str->len - l > 75) {
+			
+			if ((g_utf8_strlen (attr_str->str, -1) -l) > 75) {
+				char *p;
+
 				l += 75;
-				attr_str = g_string_insert_len (attr_str, l, CRLF " ", sizeof (CRLF " ") - 1);
+				p = g_utf8_offset_to_pointer (attr_str->str, l);
+			
+				g_string_insert_len (attr_str, (p - attr_str->str), CRLF " ", sizeof (CRLF " ") - 1);
 			}
 			else
 				break;
-		} while (l < attr_str->len);
+		} while (l < g_utf8_strlen (attr_str->str, -1));
 
-		attr_str = g_string_append (attr_str, CRLF);
+		g_string_append (attr_str, CRLF);
 
-		str = g_string_append (str, attr_str->str);
+		g_string_append (str, attr_str->str);
 		g_string_free (attr_str, TRUE);
 	}
 
-	str = g_string_append (str, "END:VCARD");
+	g_string_append (str, "END:VCARD");
 
 	return g_string_free (str, FALSE);
 }
 
+/**
+ * e_vcard_to_string:
+ * @evc: the #EVCard to export
+ * @format: the format to export to
+ *
+ * Exports @evc to a string representation, specified
+ * by the @format argument.
+ *
+ * Return value: A newly allocated string representing the vcard.
+ **/
 char*
 e_vcard_to_string (EVCard *evc, EVCardFormat format)
 {
@@ -833,6 +886,13 @@ e_vcard_to_string (EVCard *evc, EVCardFormat format)
 	}
 }
 
+/**
+ * e_vcard_dump_structure:
+ * @evc: the #EVCard to dump
+ *
+ * Prints a dump of @evc's structure to stdout. Used for
+ * debugging.
+ **/
 void
 e_vcard_dump_structure (EVCard *evc)
 {
@@ -872,7 +932,16 @@ e_vcard_dump_structure (EVCard *evc)
 	}
 }
 
-
+/**
+ * e_vcard_attribute_new:
+ * @attr_group: a group name
+ * @attr_name: an attribute name
+ *
+ * Creates a new #EVCardAttribute with the specified group and
+ * attribute names.
+ *
+ * Return value: A new #EVCardAttribute.
+ **/
 EVCardAttribute*
 e_vcard_attribute_new (const char *attr_group, const char *attr_name)
 {
@@ -886,6 +955,12 @@ e_vcard_attribute_new (const char *attr_group, const char *attr_name)
 	return attr;
 }
 
+/**
+ * e_vcard_attribute_free:
+ * @attr: attribute to free
+ *
+ * Frees an attribute, its values and its parameters.
+ **/
 void
 e_vcard_attribute_free (EVCardAttribute *attr)
 {
@@ -901,6 +976,14 @@ e_vcard_attribute_free (EVCardAttribute *attr)
 	g_free (attr);
 }
 
+/**
+ * e_vcard_attribute_copy:
+ * @attr: attribute to copy
+ *
+ * Makes a copy of @attr.
+ *
+ * Return value: A new #EVCardAttribute identical to @attr.
+ **/
 EVCardAttribute*
 e_vcard_attribute_copy (EVCardAttribute *attr)
 {
@@ -921,6 +1004,17 @@ e_vcard_attribute_copy (EVCardAttribute *attr)
 	return a;
 }
 
+/**
+ * e_vcard_remove_attributes:
+ * @evc: vcard object
+ * @attr_group: group name of attributes to be removed
+ * @attr_name: name of the arributes to be removed
+ *
+ * Removes all the attributes with group name and attribute name equal to 
+ * passed in values. If @attr_group is %NULL or an empty string,
+ * it removes all the attributes with passed in name irrespective of
+ * their group names.
+ **/
 void
 e_vcard_remove_attributes (EVCard *evc, const char *attr_group, const char *attr_name)
 {
@@ -936,7 +1030,7 @@ e_vcard_remove_attributes (EVCard *evc, const char *attr_group, const char *attr
 
 		next_attr = attr->next;
 
-		if (((!attr_group && !a->group) ||
+		if (((!attr_group || *attr_group == '\0') ||
 		     (attr_group && !g_ascii_strcasecmp (attr_group, a->group))) &&
 		    ((!attr_name && !a->name) || !g_ascii_strcasecmp (attr_name, a->name))) {
 
@@ -950,6 +1044,13 @@ e_vcard_remove_attributes (EVCard *evc, const char *attr_group, const char *attr
 	}
 }
 
+/**
+ * e_vcard_remove_attribute:
+ * @evc: an #EVCard
+ * @attr: an #EVCardAttribute to remove
+ *
+ * Removes @attr from @evc and frees it.
+ **/
 void
 e_vcard_remove_attribute (EVCard *evc, EVCardAttribute *attr)
 {
@@ -960,6 +1061,13 @@ e_vcard_remove_attribute (EVCard *evc, EVCardAttribute *attr)
 	e_vcard_attribute_free (attr);
 }
 
+/**
+ * e_vcard_add_attribute:
+ * @evc: an #EVCard
+ * @attr: an #EVCardAttribute to add
+ *
+ * Adds @attr to @evc.
+ **/
 void
 e_vcard_add_attribute (EVCard *evc, EVCardAttribute *attr)
 {
@@ -969,6 +1077,14 @@ e_vcard_add_attribute (EVCard *evc, EVCardAttribute *attr)
 	evc->priv->attributes = g_list_append (evc->priv->attributes, attr);
 }
 
+/**
+ * e_vcard_add_attribute_with_value:
+ * @evcard: an #EVCard
+ * @attr: an #EVCardAttribute to add
+ * @value: a value to assign to the attribute
+ *
+ * Adds @attr to @evcard, setting it to @value.
+ **/
 void
 e_vcard_add_attribute_with_value (EVCard *evcard,
 				  EVCardAttribute *attr, const char *value)
@@ -981,6 +1097,14 @@ e_vcard_add_attribute_with_value (EVCard *evcard,
 	e_vcard_add_attribute (evcard, attr);
 }
 
+/**
+ * e_vcard_add_attribute_with_values:
+ * @evcard: an @EVCard
+ * @attr: an #EVCardAttribute to add
+ * @...: a %NULL-terminated list of values to assign to the attribute
+ *
+ * Adds @attr to @evcard, assigning the list of values to it.
+ **/
 void
 e_vcard_add_attribute_with_values (EVCard *evcard, EVCardAttribute *attr, ...)
 {
@@ -1001,6 +1125,13 @@ e_vcard_add_attribute_with_values (EVCard *evcard, EVCardAttribute *attr, ...)
 	e_vcard_add_attribute (evcard, attr);
 }
 
+/**
+ * e_vcard_attribute_add_value:
+ * @attr: an #EVCardAttribute
+ * @value: a string value
+ *
+ * Adds @value to @attr's list of values.
+ **/
 void
 e_vcard_attribute_add_value (EVCardAttribute *attr, const char *value)
 {
@@ -1009,6 +1140,15 @@ e_vcard_attribute_add_value (EVCardAttribute *attr, const char *value)
 	attr->values = g_list_append (attr->values, g_strdup (value));
 }
 
+/**
+ * e_vcard_attribute_add_value_decoded:
+ * @attr: an #EVCardAttribute
+ * @value: an encoded value
+ * @len: the length of the encoded value, in bytes
+ *
+ * Decodes @value according to the encoding used for @attr, and
+ * adds it to @attr's list of values.
+ **/
 void
 e_vcard_attribute_add_value_decoded (EVCardAttribute *attr, const char *value, int len)
 {
@@ -1038,6 +1178,13 @@ e_vcard_attribute_add_value_decoded (EVCardAttribute *attr, const char *value, i
 	}
 }
 
+/**
+ * e_vcard_attribute_add_values:
+ * @attr: an #EVCardAttribute
+ * ...: a %NULL-terminated list of strings
+ *
+ * Adds a list of values to @attr.
+ **/
 void
 e_vcard_attribute_add_values (EVCardAttribute *attr,
 			      ...)
@@ -1062,6 +1209,12 @@ free_gstring (GString *str)
 	g_string_free (str, TRUE);
 }
 
+/**
+ * e_vcard_attribute_remove_values:
+ * @attr: an #EVCardAttribute
+ *
+ * Removes all values from @attr.
+ **/
 void
 e_vcard_attribute_remove_values (EVCardAttribute *attr)
 {
@@ -1076,6 +1229,12 @@ e_vcard_attribute_remove_values (EVCardAttribute *attr)
 	attr->decoded_values = NULL;
 }
 
+/**
+ * e_vcard_attribute_remove_params:
+ * @attr: an #EVCardAttribute
+ *
+ * Removes all parameters from @attr.
+ **/
 void
 e_vcard_attribute_remove_params (EVCardAttribute *attr)
 {
@@ -1090,6 +1249,14 @@ e_vcard_attribute_remove_params (EVCardAttribute *attr)
 	attr->encoding = EVC_ENCODING_RAW;
 }
 
+/**
+ * e_vcard_attribute_param_new:
+ * @name: the name of the new parameter
+ *
+ * Creates a new parameter named @name.
+ *
+ * Return value: A new #EVCardAttributeParam.
+ **/
 EVCardAttributeParam*
 e_vcard_attribute_param_new (const char *name)
 {
@@ -1099,6 +1266,12 @@ e_vcard_attribute_param_new (const char *name)
 	return param;
 }
 
+/**
+ * e_vcard_attribute_param_free:
+ * @param: an #EVCardAttributeParam
+ *
+ * Frees @param and its values.
+ **/
 void
 e_vcard_attribute_param_free (EVCardAttributeParam *param)
 {
@@ -1111,6 +1284,14 @@ e_vcard_attribute_param_free (EVCardAttributeParam *param)
 	g_free (param);
 }
 
+/**
+ * e_vcard_attribute_param_copy:
+ * @param: an #EVCardAttributeParam
+ *
+ * Makes a copy of @param.
+ *
+ * Return value: a new #EVCardAttributeParam identical to @param.
+ **/
 EVCardAttributeParam*
 e_vcard_attribute_param_copy (EVCardAttributeParam *param)
 {
@@ -1128,6 +1309,13 @@ e_vcard_attribute_param_copy (EVCardAttributeParam *param)
 	return p;
 }
 
+/**
+ * e_vcard_attribute_add_param:
+ * @attr: an #EVCardAttribute
+ * @param: an #EVCardAttributeParam to add
+ *
+ * Adds @param to @attr's list of parameters.
+ **/
 void
 e_vcard_attribute_add_param (EVCardAttribute *attr,
 			     EVCardAttributeParam *param)
@@ -1146,7 +1334,8 @@ e_vcard_attribute_add_param (EVCardAttribute *attr,
 		}
 
 		if (param->values && param->values->data) {
-			if (!g_ascii_strcasecmp ((char*)param->values->data, "b"))
+			if (!g_ascii_strcasecmp ((char*)param->values->data, "b") ||
+			    !g_ascii_strcasecmp ((char*)param->values->data, "BASE64"))
 				attr->encoding = EVC_ENCODING_BASE64;
 			else if (!g_ascii_strcasecmp ((char*)param->values->data, EVC_QUOTEDPRINTABLE))
 				attr->encoding = EVC_ENCODING_QP;
@@ -1163,6 +1352,13 @@ e_vcard_attribute_add_param (EVCardAttribute *attr,
 	}
 }
 
+/**
+ * e_vcard_attribute_param_add_value:
+ * @param: an #EVCardAttributeParam
+ * @value: a string value to add
+ *
+ * Adds @value to @param's list of values.
+ **/
 void
 e_vcard_attribute_param_add_value (EVCardAttributeParam *param,
 				   const char *value)
@@ -1172,6 +1368,13 @@ e_vcard_attribute_param_add_value (EVCardAttributeParam *param,
 	param->values = g_list_append (param->values, g_strdup (value));
 }
 
+/**
+ * e_vcard_attribute_param_add_values:
+ * @param: an #EVCardAttributeParam
+ * ...: a %NULL-terminated list of strings
+ *
+ * Adds a list of values to @param.
+ **/
 void
 e_vcard_attribute_param_add_values (EVCardAttributeParam *param,
 				    ...)
@@ -1190,6 +1393,14 @@ e_vcard_attribute_param_add_values (EVCardAttributeParam *param,
 	va_end (ap);
 }
 
+/**
+ * e_vcard_attribute_add_param_with_value:
+ * @attr: an #EVCardAttribute
+ * @param: an #EVCardAttributeParam
+ * @value: a string value
+ *
+ * Adds @value to @param, then adds @param to @attr.
+ **/
 void
 e_vcard_attribute_add_param_with_value (EVCardAttribute *attr,
 					EVCardAttributeParam *param, const char *value)
@@ -1202,6 +1413,15 @@ e_vcard_attribute_add_param_with_value (EVCardAttribute *attr,
 	e_vcard_attribute_add_param (attr, param);
 }
 
+/**
+ * e_vcard_attribute_add_param_with_values:
+ * @attr: an #EVCardAttribute
+ * @param: an #EVCardAttributeParam
+ * ...: a %NULL-terminated list of strings
+ *
+ * Adds the list of values to @param, then adds @param
+ * to @attr.
+ **/
 void
 e_vcard_attribute_add_param_with_values (EVCardAttribute *attr,
 					 EVCardAttributeParam *param, ...)
@@ -1223,6 +1443,12 @@ e_vcard_attribute_add_param_with_values (EVCardAttribute *attr,
 	e_vcard_attribute_add_param (attr, param);
 }
 
+/**
+ * e_vcard_attribute_param_remove_values:
+ * @param: an #EVCardAttributeParam
+ *
+ * Removes and frees all values from @param.
+ **/
 void
 e_vcard_attribute_param_remove_values (EVCardAttributeParam *param)
 {
@@ -1233,6 +1459,15 @@ e_vcard_attribute_param_remove_values (EVCardAttributeParam *param)
 	param->values = NULL;
 }
 
+/**
+ * e_vcard_get_attributes:
+ * @evcard: an #EVCard
+ *
+ * Gets the list of attributes from @evcard. The list and its
+ * contents are owned by @evcard, and must not be freed.
+ *
+ * Return value: A list of attributes of type #EVCardAttribute.
+ **/
 GList*
 e_vcard_get_attributes (EVCard *evcard)
 {
@@ -1241,6 +1476,14 @@ e_vcard_get_attributes (EVCard *evcard)
 	return evcard->priv->attributes;
 }
 
+/**
+ * e_vcard_attribute_get_group:
+ * @attr: an #EVCardAttribute
+ *
+ * Gets the group name of @attr.
+ *
+ * Return value: The attribute's group name.
+ **/
 const char*
 e_vcard_attribute_get_group (EVCardAttribute *attr)
 {
@@ -1249,6 +1492,14 @@ e_vcard_attribute_get_group (EVCardAttribute *attr)
 	return attr->group;
 }
 
+/**
+ * e_vcard_attribute_get_name:
+ * @attr: an #EVCardAttribute
+ *
+ * Gets the name of @attr.
+ *
+ * Return value: The attribute's name.
+ **/
 const char*
 e_vcard_attribute_get_name (EVCardAttribute *attr)
 {
@@ -1257,6 +1508,15 @@ e_vcard_attribute_get_name (EVCardAttribute *attr)
 	return attr->name;
 }
 
+/**
+ * e_vcard_attribute_get_values:
+ * @attr: an #EVCardAttribute
+ *
+ * Gets the list of values from @attr. The list and its
+ * contents are owned by @attr, and must not be freed.
+ * 
+ * Return value: A list of string values.
+ **/
 GList*
 e_vcard_attribute_get_values (EVCardAttribute *attr)
 {
@@ -1265,6 +1525,16 @@ e_vcard_attribute_get_values (EVCardAttribute *attr)
 	return attr->values;
 }
 
+/**
+ * e_vcard_attribute_get_values_decoded:
+ * @attr: an #EVCardAttribute
+ *
+ * Gets the list of values from @attr, decoding them if
+ * necessary. The list and its contents are owned by @attr,
+ * and must not be freed.
+ *
+ * Return value: A list of values of type #GString.
+ **/
 GList*
 e_vcard_attribute_get_values_decoded (EVCardAttribute *attr)
 {
@@ -1294,6 +1564,14 @@ e_vcard_attribute_get_values_decoded (EVCardAttribute *attr)
 	return attr->decoded_values;
 }
 
+/**
+ * e_vcard_attribute_is_single_valued:
+ * @attr: an #EVCardAttribute
+ *
+ * Checks if @attr has a single value.
+ *
+ * Return value: %TRUE if the attribute has exactly one value, %FALSE otherwise.
+ **/
 gboolean
 e_vcard_attribute_is_single_valued (EVCardAttribute *attr)
 {
@@ -1306,6 +1584,14 @@ e_vcard_attribute_is_single_valued (EVCardAttribute *attr)
 	return TRUE;
 }
 
+/**
+ * e_vcard_attribute_get_value:
+ * @attr: an #EVCardAttribute
+ *
+ * Gets the value of a single-valued #EVCardAttribute, @attr.
+ *
+ * Return value: A newly allocated string representing the value.
+ **/
 char*
 e_vcard_attribute_get_value (EVCardAttribute *attr)
 {
@@ -1321,6 +1607,15 @@ e_vcard_attribute_get_value (EVCardAttribute *attr)
 	return values ? g_strdup ((char*)values->data) : NULL;
 }
 
+/**
+ * e_vcard_attribute_get_value_decoded:
+ * @attr: an #EVCardAttribute
+ *
+ * Gets the value of a single-valued #EVCardAttribute, @attr, decoding
+ * it if necessary.
+ *
+ * Return value: A newly allocated #GString representing the value.
+ **/
 GString*
 e_vcard_attribute_get_value_decoded (EVCardAttribute *attr)
 {
@@ -1340,6 +1635,15 @@ e_vcard_attribute_get_value_decoded (EVCardAttribute *attr)
 	return str ? g_string_new_len (str->str, str->len) : NULL;
 }
 
+/**
+ * e_vcard_attribute_has_type:
+ * @attr: an #EVCardAttribute
+ * @typestr: a string representing the type
+ *
+ * Checks if @attr has an #EVCardAttributeParam of the specified type.
+ *
+ * Return value: %TRUE if such a parameter exists, %FALSE otherwise.
+ **/
 gboolean
 e_vcard_attribute_has_type (EVCardAttribute *attr, const char *typestr)
 {
@@ -1354,12 +1658,12 @@ e_vcard_attribute_has_type (EVCardAttribute *attr, const char *typestr)
 	for (p = params; p; p = p->next) {
 		EVCardAttributeParam *param = p->data;
 
-		if (!strcasecmp (e_vcard_attribute_param_get_name (param), EVC_TYPE)) {
+		if (!g_ascii_strcasecmp (e_vcard_attribute_param_get_name (param), EVC_TYPE)) {
 			GList *values = e_vcard_attribute_param_get_values (param);
 			GList *v;
 
 			for (v = values; v; v = v->next) {
-				if (!strcasecmp ((char*)v->data, typestr))
+				if (!g_ascii_strcasecmp ((char*)v->data, typestr))
 					return TRUE;
 			}
 		}
@@ -1368,6 +1672,15 @@ e_vcard_attribute_has_type (EVCardAttribute *attr, const char *typestr)
 	return FALSE;
 }
 
+/**
+ * e_vcard_attribute_get_params:
+ * @attr: an #EVCardAttribute
+ * 
+ * Gets the list of parameters from @attr. The list and its
+ * contents are owned by @attr, and must not be freed.
+ *
+ * Return value: A list of elements of type #EVCardAttributeParam.
+ **/
 GList*
 e_vcard_attribute_get_params (EVCardAttribute *attr)
 {
@@ -1376,6 +1689,14 @@ e_vcard_attribute_get_params (EVCardAttribute *attr)
 	return attr->params;
 }
 
+/**
+ * e_vcard_attribute_param_get_name:
+ * @param: an #EVCardAttributeParam
+ *
+ * Gets the name of @param.
+ *
+ * Return value: The name of the parameter.
+ **/
 const char*
 e_vcard_attribute_param_get_name (EVCardAttributeParam *param)
 {
@@ -1384,6 +1705,15 @@ e_vcard_attribute_param_get_name (EVCardAttributeParam *param)
 	return param->name;
 }
 
+/**
+ * e_vcard_attribute_param_get_values:
+ * @param: an #EVCardAttributeParam
+ * 
+ * Gets the list of values from @param. The list and its
+ * contents are owned by @param, and must not be freed.
+ *
+ * Return value: A list of string elements representing the parameter's values.
+ **/
 GList*
 e_vcard_attribute_param_get_values (EVCardAttributeParam *param)
 {
