@@ -214,7 +214,7 @@ groupwise_populate_details_from_item (CamelMimeMessage *msg, EGwItem *item)
 	temp_str = (char *)e_gw_item_get_subject(item);
 	if(temp_str)
 		camel_mime_message_set_subject (msg, temp_str);
-	dtstring = e_gw_item_get_creation_date (item);
+	dtstring = e_gw_item_get_delivered_date (item);
 	if(dtstring) {
 		int offset = 0;
 		time_t time = e_gw_connection_get_date_from_string (dtstring);
@@ -718,6 +718,7 @@ camel_gw_folder_new(CamelStore *store, const char *folder_name, const char *fold
 
 	journal_file = g_strdup_printf ("%s/journal",folder_dir);
 	gw_folder->journal = camel_groupwise_journal_new (gw_folder, journal_file);
+	g_free (journal_file);
 	if (!gw_folder->journal) {
 		camel_object_unref (folder);
 		return NULL;
@@ -881,7 +882,7 @@ groupwise_refresh_folder(CamelFolder *folder, CamelException *ex)
 		goto end1;
 
 	if (!strcmp (folder->full_name, "Trash") || is_proxy) {
-		status = e_gw_connection_get_items (cnc, container_id, "peek recipient distribution created attachments subject status size", NULL, &list);
+		status = e_gw_connection_get_items (cnc, container_id, "peek recipient distribution delivered attachments subject status size", NULL, &list);
 		if (status != E_GW_CONNECTION_STATUS_OK) {
 			if (status ==E_GW_CONNECTION_STATUS_OTHER) {
 				g_warning ("Trash full....Empty Trash!!!!\n");
@@ -1140,7 +1141,7 @@ gw_update_cache ( CamelFolder *folder, GList *list, CamelException *ex)
 				mi->info.date_sent = mi->info.date_received = actual_time;
 			}
 		} else {
-			temp_date = e_gw_item_get_creation_date(item);
+			temp_date = e_gw_item_get_delivered_date(item);
 			if (temp_date) {
 				time_t time = e_gw_connection_get_date_from_string (temp_date);
 				time_t actual_time = camel_header_decode_date (ctime(&time), NULL);
@@ -1317,7 +1318,7 @@ gw_update_summary ( CamelFolder *folder, GList *list,CamelException *ex)
 				mi->info.date_sent = mi->info.date_received = actual_time;
 			} 
 		} else {
-			temp_date = e_gw_item_get_creation_date(item);
+			temp_date = e_gw_item_get_delivered_date(item);
 			if (temp_date) {
 				time_t time = e_gw_connection_get_date_from_string (temp_date);
 				time_t actual_time = camel_header_decode_date (ctime(&time), NULL);
@@ -1774,7 +1775,7 @@ groupwise_transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 			if (!strcmp(source->full_name, JUNK_FOLDER))
 				camel_folder_set_message_flags (source, old_uid, CAMEL_GW_MESSAGE_JUNK|CAMEL_MESSAGE_JUNK|CAMEL_MESSAGE_JUNK_LEARN, 0);
 			else if (!strcmp(destination->full_name, JUNK_FOLDER))
-				camel_folder_set_message_flags (source, old_uid, CAMEL_MESSAGE_JUNK, CAMEL_GW_MESSAGE_JUNK);
+				camel_folder_set_message_flags (source, old_uid, CAMEL_MESSAGE_JUNK|CAMEL_GW_MESSAGE_JUNK, CAMEL_GW_MESSAGE_JUNK);
 
 			CamelGroupwiseMessageInfo *dest_info = (CamelGroupwiseMessageInfo *)camel_message_info_clone((CamelMessageInfo *)src_info);
 			tmp = g_strsplit (old_uid, ":", -1);
