@@ -5,7 +5,7 @@
  *  Authors: Michael Zucchi <notzed@ximian.com>
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,7 +13,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
@@ -25,7 +25,6 @@
 
 #include <ctype.h>
 #include <sys/stat.h>
-#include <sys/uio.h>
 #include <unistd.h>
 #include <errno.h>
 #include <string.h>
@@ -555,13 +554,13 @@ local_summary_decode_x_evolution(CamelLocalSummary *cls, const char *xev, CamelL
 		params = camel_header_param_list_decode(header+1);
 		scan = params;
 		while (scan) {
-			if (!strcasecmp(scan->name, "flags")) {
+			if (!g_ascii_strcasecmp(scan->name, "flags")) {
 				char **flagv = g_strsplit(scan->value, ",", 1000);
 
 				for (i=0;flagv[i];i++)
 					camel_message_info_set_user_flag((CamelMessageInfo *)mi, flagv[i], TRUE);
 				g_strfreev(flagv);
-			} else if (!strcasecmp(scan->name, "tags")) {
+			} else if (!g_ascii_strcasecmp(scan->name, "tags")) {
 				char **tagv = g_strsplit(scan->value, ",", 10000);
 				char *val;
 
