@@ -5,7 +5,7 @@
  * Copyright (C) 2004 Novell, Inc.
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,7 +13,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
@@ -37,17 +37,26 @@
 #include <libedataserverui/e-contact-store.h>
 #include <libedataserverui/e-book-auth-util.h>
 #include <libedataserver/e-sexp.h>
+#include <libedataserver/e-categories.h>
 #include "e-name-selector-dialog.h"
 
 typedef struct {
 	gchar        *name;
 
 	GtkBox       *section_box;
+	GtkLabel     *label;  
 	GtkButton    *transfer_button;
+	GtkButton    *remove_button;
 	GtkTreeView  *destination_view;
 }
 Section;
 
+typedef struct {
+	GtkTreeView *view;
+	GtkButton   *button;
+	ENameSelectorDialog *dlg_ptr;
+} SelData;
+
 static ESource *find_first_source             (ESourceList *source_list);
 static void     search_changed                (ENameSelectorDialog *name_selector_dialog);
 static void     source_selected               (ENameSelectorDialog *name_selector_dialog, ESource *source);
@@ -58,6 +67,7 @@ static void     contact_activated             (ENameSelectorDialog *name_selecto
 static void     destination_activated         (ENameSelectorDialog *name_selector_dialog, GtkTreePath *path,
 					       GtkTreeViewColumn *column, GtkTreeView *tree_view);
 static gboolean destination_key_press         (ENameSelectorDialog *name_selector_dialog, GdkEventKey *event, GtkTreeView *tree_view);
+static void remove_button_clicked (GtkButton *button, SelData *data);
 static void     remove_books                  (ENameSelectorDialog *name_selector_dialog);
 static void     contact_column_formatter      (GtkTreeViewColumn *column, GtkCellRenderer *cell,
 					       GtkTreeModel *model, GtkTreeIter *iter,
@@ -84,6 +94,43 @@ e_name_selector_dialog_set_property (GObject *object, guint prop_id,
 {
 }
 
+/* FIXME: category_list should become part of ENameSelectorDialog structure */
+GList *category_list;
+
+static int
+compare_func (const void *voida, const void *voidb)
+{
+	const char *stringa = voida;
+        const char *stringb = voidb;
+	return g_utf8_collate (stringa, stringb);
+}
+
+static void
+e_name_selector_dialog_populate_categories (ENameSelectorDialog *name_selector_dialog)
+{
+	GtkWidget *category_option_menu;
+	GtkWidget *category_menu;
+	GList *l;
+	category_option_menu = glade_xml_get_widget (name_selector_dialog->gui, "optionmenu-category");
+
+	category_list = e_categories_get_list () ;
+	category_list = g_list_prepend (category_list, "Any Category");
+	g_list_sort (category_list, compare_func);
+
+	category_menu = gtk_menu_new ();
+	l = category_list;
+	while (l) {	
+		GtkWidget *item;
+		item = gtk_menu_item_new_with_label (l->data);
+		gtk_menu_shell_append (GTK_MENU_SHELL (category_menu), item);
+		l = l->next;
+	}
+	gtk_widget_show_all (category_menu);
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (category_option_menu), category_menu);
+	
+	g_signal_connect_swapped (category_option_menu, "changed", G_CALLBACK (search_changed), name_selector_dialog);
+}
+
 static void
 e_name_selector_dialog_init (ENameSelectorDialog *name_selector_dialog)
 {
@@ -97,7 +144,7 @@ e_name_selector_dialog_init (ENameSelectorDialog *name_selector_dialog)
 
 	/* Get Glade GUI */
 
-	name_selector_dialog->gui = glade_xml_new (E_DATA_SERVER_UI_GLADEDIR "/e-name-selector-dialog.glade", NULL, NULL);
+	name_selector_dialog->gui = glade_xml_new (E_DATA_SERVER_UI_GLADEDIR "/e-name-selector-dialog.glade", NULL, GETTEXT_PACKAGE);
 
 	widget = glade_xml_get_widget (name_selector_dialog->gui, "name-selector-box");
 	if (!widget) {
@@ -176,6 +223,8 @@ e_name_selector_dialog_init (ENameSelectorDialog *name_selector_dialog)
 	container = glade_xml_get_widget (name_selector_dialog->gui, "source-menu-box");
 	gtk_box_pack_start (GTK_BOX (container), widget, TRUE, TRUE, 0);
 
+	e_name_selector_dialog_populate_categories (name_selector_dialog);
+
 	/* Set up search-as-you-type signal */
 
 	widget = glade_xml_get_widget (name_selector_dialog->gui, "search");
@@ -201,7 +250,7 @@ e_name_selector_dialog_init (ENameSelectorDialog *name_selector_dialog)
 	gtk_container_set_border_width  (GTK_CONTAINER (name_selector_dialog), 4);
 	gtk_window_set_title            (GTK_WINDOW (name_selector_dialog), _("Select Contacts from Address Book"));
 }
-
+	
 /* Partial, repeatable destruction. Release references. */
 static void
 e_name_selector_dialog_dispose (GObject *object)
@@ -213,7 +262,7 @@ e_name_selector_dialog_dispose (GObject *object)
 					      name_selector_dialog);
 
 	remove_books (name_selector_dialog);
-
+	
 	if (G_OBJECT_CLASS (e_name_selector_dialog_parent_class)->dispose)
 		G_OBJECT_CLASS (e_name_selector_dialog_parent_class)->dispose (object);
 }
@@ -223,6 +272,7 @@ static void
 e_name_selector_dialog_finalize (GObject *object)
 {
 	/* TODO: Free stuff */
+	g_list_free (category_list);
 
 	if (G_OBJECT_CLASS (e_name_selector_dialog_parent_class)->finalize)
 		G_OBJECT_CLASS (e_name_selector_dialog_parent_class)->finalize (object);
@@ -402,6 +452,17 @@ find_section_by_name (ENameSelectorDialog *name_selector_dialog, const gchar *na
 	return -1;
 }
 
+static void
+selection_changed (GtkTreeSelection *selection, SelData *data)
+{
+	GtkTreeSelection *contact_selection;
+	gboolean          have_selection;
+		
+	contact_selection = gtk_tree_view_get_selection (data->view);
+	have_selection = gtk_tree_selection_get_selected (contact_selection, NULL, NULL);
+	gtk_widget_set_sensitive (GTK_WIDGET (data->button), have_selection);
+}
+
 static gint
 add_section (ENameSelectorDialog *name_selector_dialog,
 	     const gchar *name, const gchar *pretty_name, EDestinationStore *destination_store)
@@ -409,8 +470,11 @@ add_section (ENameSelectorDialog *name_selector_dialog,
 	Section            section;
 	GtkTreeViewColumn *column;
 	GtkCellRenderer   *cell_renderer;
-	GtkWidget         *widget;
-	gchar		  *text;
+	GtkWidget  	  *vbox, *hbox, *chbox;
+	GtkWidget 	  *widget, *image, *label;
+	SelData		  *data;
+	GtkTreeSelection  *selection;
+	gchar 		  *text;
 
 	g_assert (name != NULL);
 	g_assert (pretty_name != NULL);
@@ -420,7 +484,9 @@ add_section (ENameSelectorDialog *name_selector_dialog,
 
 	section.name = g_strdup (name);
 	section.section_box      = GTK_BOX (gtk_hbox_new (FALSE, 12));
-	section.transfer_button  = GTK_BUTTON (gtk_button_new_with_mnemonic (pretty_name));
+	section.label = GTK_LABEL (gtk_label_new_with_mnemonic (pretty_name));
+	section.transfer_button  = GTK_BUTTON (gtk_button_new());
+	section.remove_button  = GTK_BUTTON (gtk_button_new());
 	section.destination_view = GTK_TREE_VIEW (gtk_tree_view_new ());
 
 	if (pango_parse_markup (pretty_name, -1, '_', NULL,
@@ -433,7 +499,18 @@ add_section (ENameSelectorDialog *name_selector_dialog,
 	/* Set up transfer button */
 	g_signal_connect_swapped (section.transfer_button, "clicked",
 				  G_CALLBACK (transfer_button_clicked), name_selector_dialog);
+	
+	/*data for the remove callback*/
+	data = g_malloc0(sizeof(SelData));
+	data->view = section.destination_view;
+	data->dlg_ptr = name_selector_dialog;
 
+	/*Associate to an object destroy so that it gets freed*/
+	g_object_set_data_full ((GObject *)section.destination_view, "sel-remove-data", data, g_free);
+	
+	g_signal_connect(section.remove_button, "clicked",
+				  G_CALLBACK (remove_button_clicked), data);
+	
 	/* Set up view */
 	column = gtk_tree_view_column_new ();
 	cell_renderer = GTK_CELL_RENDERER (gtk_cell_renderer_text_new ());
@@ -445,18 +522,77 @@ add_section (ENameSelectorDialog *name_selector_dialog,
 	gtk_tree_view_set_headers_visible (section.destination_view, FALSE);
 	gtk_tree_view_set_model (section.destination_view, GTK_TREE_MODEL (destination_store));
 
+	vbox = gtk_vbox_new (FALSE, 0);
+	chbox = gtk_hbox_new (FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (chbox), vbox, FALSE, FALSE, 12);
+
 	/* Pack button (in an alignment) */
 	widget = gtk_alignment_new (0.5, 0.0, 0.0, 0.0);
 	gtk_container_add (GTK_CONTAINER (widget), GTK_WIDGET (section.transfer_button));
-	gtk_box_pack_start (section.section_box, widget, FALSE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, TRUE, 6);
 	gtk_size_group_add_widget (name_selector_dialog->button_size_group, GTK_WIDGET (section.transfer_button));
+	
+	/*to get the image embedded in the button*/
+	widget = gtk_alignment_new (0.7, 0.5, 0.0, 0.0);
+	gtk_container_add (GTK_CONTAINER (section.transfer_button), GTK_WIDGET (widget));
+
+	hbox = gtk_hbox_new (FALSE, 2);
+	gtk_widget_show (GTK_WIDGET(hbox));
+	gtk_container_add (GTK_CONTAINER (widget), GTK_WIDGET(hbox));
+
+	label = gtk_label_new_with_mnemonic (_("_Add"));
+	gtk_widget_show (label);
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+
+	image = gtk_image_new_from_stock ("gtk-go-forward", GTK_ICON_SIZE_BUTTON);
+	gtk_widget_show (image);
+	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+	
+	widget = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+	gtk_container_add (GTK_CONTAINER (widget), GTK_WIDGET (section.remove_button));
+	gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, TRUE, 0);
+	gtk_size_group_add_widget (name_selector_dialog->button_size_group, GTK_WIDGET (section.remove_button));
+	gtk_widget_set_sensitive (GTK_WIDGET (section.remove_button), FALSE);
+
+	widget = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+	gtk_container_add (GTK_CONTAINER (section.remove_button), widget);
+
+	hbox = gtk_hbox_new (FALSE, 2);
+	gtk_widget_show (hbox);
+	gtk_container_add (GTK_CONTAINER (widget), hbox);
+
+	image = gtk_image_new_from_stock ("gtk-go-back", GTK_ICON_SIZE_BUTTON);
+	gtk_widget_show (image);
+	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+
+	label = gtk_label_new_with_mnemonic (_("_Remove"));
+	gtk_widget_show (label);
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+
+	vbox = gtk_vbox_new (FALSE, 0);
+	
+	widget = gtk_alignment_new (0.5, 0.0, 0.0, 0.0);
+	gtk_container_add (GTK_CONTAINER (widget), GTK_WIDGET (section.label));
+	gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, TRUE, 0);
 
 	/* Pack view (in a scrolled window) */
 	widget = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (widget), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (widget), GTK_SHADOW_IN);
 	gtk_container_add (GTK_CONTAINER (widget), GTK_WIDGET (section.destination_view));
-	gtk_box_pack_start (section.section_box, widget, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (chbox), widget, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), chbox, TRUE, TRUE, 0);
+	gtk_box_pack_start (section.section_box, vbox, TRUE, TRUE, 0);
+
+	/*data for 'changed' callback*/
+	data = g_malloc0(sizeof(SelData));
+	data->view = section.destination_view;
+	data->button = section.remove_button;
+	g_object_set_data_full ((GObject *)section.destination_view, "sel-change-data", data, g_free);
+	selection = gtk_tree_view_get_selection(section.destination_view);
+	g_signal_connect(selection, "changed",
+				  G_CALLBACK (selection_changed), data);
+
 	g_signal_connect_swapped (section.destination_view, "row-activated",
 				  G_CALLBACK (destination_activated), name_selector_dialog);
 	g_signal_connect_swapped (section.destination_view, "key-press-event",
@@ -556,20 +692,45 @@ search_changed (ENameSelectorDialog *name_selector_dialog)
 {
 	EContactStore *contact_store;
 	EBookQuery    *book_query;
+	GtkWidget     *category_option_menu;
 	const gchar   *text;
+	gint 	      category_id;	
 	gchar         *text_escaped;
 	gchar         *query_string;
+	const gchar   *category;
+	gchar         *category_escaped;
+
+	category_option_menu = glade_xml_get_widget(name_selector_dialog->gui, "optionmenu-category");
+	category_id = gtk_option_menu_get_history (GTK_OPTION_MENU(category_option_menu));
+	category = g_list_nth_data (category_list, category_id);
+	category_escaped = escape_sexp_string (category);
+
+	if (!category)
+		return;
 
 	text = gtk_entry_get_text (name_selector_dialog->search_entry);
 	text_escaped = escape_sexp_string (text);
-	query_string = g_strdup_printf ("(or (beginswith \"file_as\" %s) "
-					"    (beginswith \"full_name\" %s) "
-					"    (beginswith \"email\" %s) "
-					"    (beginswith \"nickname\" %s))",
-					text_escaped, text_escaped, text_escaped, text_escaped);
+
+	if ( !strcmp (category,"Any Category")) {
+		query_string = g_strdup_printf ("(or (beginswith \"file_as\" %s) "
+						"    (beginswith \"full_name\" %s) "
+						"    (beginswith \"email\" %s) "
+						"    (beginswith \"nickname\" %s)))",
+						text_escaped, text_escaped, text_escaped, text_escaped);
+	}
+	else {
+		query_string = g_strdup_printf ("(and (is \"category_list\" %s) "
+						"(or (beginswith \"file_as\" %s) "
+						"    (beginswith \"full_name\" %s) "
+						"    (beginswith \"email\" %s) "
+						"    (beginswith \"nickname\" %s)))",
+						category_escaped,text_escaped, text_escaped, text_escaped, text_escaped);
+	}
+
 	book_query = e_book_query_from_string (query_string);
 	g_free (query_string);
 	g_free (text_escaped);
+	g_free (category_escaped);
 
 	contact_store = e_name_selector_model_peek_contact_store (name_selector_dialog->name_selector_model);
 	e_contact_store_set_query (contact_store, book_query);
@@ -671,8 +832,7 @@ destination_activated (ENameSelectorDialog *name_selector_dialog, GtkTreePath *p
 }
 
 static gboolean 
-destination_key_press (ENameSelectorDialog *name_selector_dialog, 
-		       GdkEventKey *event, GtkTreeView *tree_view)
+remove_selection (ENameSelectorDialog *name_selector_dialog, GtkTreeView *tree_view)
 {
 	gint               section_index;
 	EDestinationStore *destination_store;
@@ -680,10 +840,6 @@ destination_key_press (ENameSelectorDialog *name_selector_dialog,
 	Section           *section;
 	GtkTreeIter        iter;
 
-	/* we only care about DEL key */
-	if (event->keyval != GDK_Delete)
-		return FALSE;
-
 	section_index = find_section_by_tree_view (name_selector_dialog, tree_view);
 	if (section_index < 0) {
 		g_warning ("ENameSelectorDialog got key press from unknown view!");
@@ -708,6 +864,29 @@ destination_key_press (ENameSelectorDialog *name_selector_dialog,
 	return TRUE;
 }
 
+static void
+remove_button_clicked (GtkButton *button, SelData *data)
+{
+	GtkTreeView *view;
+	ENameSelectorDialog *name_selector_dialog;
+	
+	view = data->view; 
+	name_selector_dialog = data->dlg_ptr;
+	remove_selection (name_selector_dialog, view);
+}
+	
+static gboolean 
+destination_key_press (ENameSelectorDialog *name_selector_dialog, 
+		       GdkEventKey *event, GtkTreeView *tree_view)
+{
+
+	/* we only care about DEL key */
+	if (event->keyval != GDK_Delete)
+		return FALSE;
+	return remove_selection (name_selector_dialog, tree_view);
+
+}
+
 static void
 transfer_button_clicked (ENameSelectorDialog *name_selector_dialog, GtkButton *transfer_button)
 {
