@@ -7,15 +7,15 @@
  * Copyright 2003, Novell, Inc.
  *
  * This program is free software; you can redistribute it and/or 
- * modify it under the terms of version 2 of the GNU General Public 
+ * modify it under the terms of version 2 of the GNU Lesser General Public 
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
@@ -468,7 +468,7 @@ set_status_tracking_changes (SoupSoapMessage *msg, EGwSendOptionsStatusTracking
 		comp_name = g_strconcat (comp, "StatusInfo", NULL);
 		e_gw_sendoptions_write_settings (msg, comp_name, value, NULL, TRUE);
 		g_free (comp_name), comp_name = NULL;
-		
+		g_free (value), value = NULL;
 	}
 		
 	if (!strcmp (comp, "mail")) {
@@ -480,6 +480,7 @@ set_status_tracking_changes (SoupSoapMessage *msg, EGwSendOptionsStatusTracking
 			comp_name = g_strconcat (comp, "AutoDelete", NULL);
 			e_gw_sendoptions_write_settings (msg, comp_name, value, NULL, TRUE);
 			g_free (comp_name), comp_name = NULL;
+			g_free (value), value = NULL;
 		}
 	}
 
